import {
  CollapseCard, Select, Text, Date as DateInput,
} from '@zeedhi/common';
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class FCMAnaliseNaoConformidade {
  public setTypeLocal({ component }:{ component:Select }) {
    Http.get('/getLocalType').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.options.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMTIPOLOCALCLI,
          value: element.NRTIPOLOCALCLI,
        });
      });
    });
  }

  public setLocal({ component }:{ component:Select }) {
    Http.get('/getLocal').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.local.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMLOCALCLIENTE,
          value: element.NRLOCALCLIENTE,
        });
      });
    });
  }

  public setSector({ component }:{ component:Select }) {
    Http.get('/getSector').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.options.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMSETOR,
          value: element.NRSETOR,
        });
      });
    });
  }

  public getAllData() {
    let dataDe = Metadata.getInstance<DateInput>('DataInicioPeriodo').value;
    let dataAte = Metadata.getInstance<DateInput>('DataFimPeriodo').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    const test = Metadata.getInstance<Select>('SelectLocal').value;
    return {
      LocalType: Metadata.getInstance<Select>('SelectTipoLocal').value,
      Local: test,
      Sector: Metadata.getInstance<Select>('SelectSetor').value,
      PeriodoDe: dataDe,
      PeriodoAte: dataAte,
    };
  }

  public getDataCards() {
    Http.post('/getAnaliseCriterio', this.getAllData()).then((response: IDictionary) => {
      const dados = response.data.dataset.criteriosConformidade;
      const iterable = Metadata.getInstance<IterableComponentRender>('iterable_CardsDeLocal');
      iterable.reload();
      iterable.datasource.data = [];
      Object.keys(dados).forEach((element: string) => {
        iterable.datasource.data.push({
          Local: element,
          Conformidades: `${dados[element].length}`,
          dadosFilho: dados[element],
        });
      });
    });
  }

  public filterPeriodo() {
    let dataDe = Metadata.getInstance<DateInput>('DataInicioPeriodoCriterio').value;
    let dataAte = Metadata.getInstance<DateInput>('DataFimPeriodoCriterio').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    return {
      PeriodoDe: dataDe,
      PeriodoAte: dataAte,
    };
  }

  public principaisNaoConformidade() {
    const iterable = Metadata.getInstance<IterableComponentRender>('iterable_Cards');
    iterable.datasource.data = [];
    Http.post('/getPrincipaisCriterio', this.filterPeriodo()).then((response: IDictionary) => {
      response.data.dataset.principaisConformidade.forEach((element: IDictionary) => {
        iterable.datasource.data.push({
          tipo: element.NMCRITERIOTARGERCHK,
          quantidade: element.QTDADE,
        });
      });
    });
  }

  public expandCard({ component }:{ component:Text }) {
    component.text = (component.text === 'ver mais ') ? 'ver menos' : 'ver mais ';
    const nomeComponent = component.name.split('_')[1];
    const card = Metadata.getInstance<CollapseCard>(`Cards_${nomeComponent}`);
    card.expanded = !card.expanded;
  }

  public getDateFather({ component }:{ component:IterableComponentRender }) {
    const iterable = Metadata.getInstance<IterableComponentRender>('iterable_CardsDeLocal');
    const nomeComponent = component.name.split('_')[1];
    const valores = iterable.datasource.data.filter((dados: IDictionary) => dados.Local === nomeComponent);
    valores[0].dadosFilho.forEach((element: IDictionary) => {
      component.datasource.data.push({
        conformidades: element,
      });
    });
  }
}
