import {
  Metadata, Http,
  IDictionary, Router,
  IEventParam,
} from '@zeedhi/core';
import {
  Select, Switch,
  Container, Form, TextInput,
  Button, Text,
} from '@zeedhi/common';
import { getRouteParams } from '../router';

export class LocalAtendimentoController {
  public dataValue: string = '';

  public editable = false;

  // Lista para cada campo da tela
  public localDataList: IDictionary = {
    localTypeLocalAt: [],
    localParentLocalAt: [],
    sectorLocalAt: [],
    group: [],
    employee: [],
    checkListLocalAt: [],
  };

  // Inicializar os meus dados
  public inicializarDados() {
    Promise.all([this.setLocal(), this.setLocalParent(), this.setSector(), this.setCheckList()]).then(() => {
      this.getToUpdate();
    });
    // this.setGroup();
    // this.setEmployee();
  }

  // Setar dados dentro da minha lista
  public setDados(nome: string, code:number) {
    return {
      name: nome,
      value: code,
    };
  }

  // Seta dados especificamente para o campo "Tipo de Local"
  public setDadosLocal(nome: string, code:number, dif:string) {
    return {
      name: nome,
      value: code,
      addInfo: dif,
    };
  }

  // Habilita o campo checklist
  public validaCheckListPadrao({ component }: IEventParam<Select>) {
    const checkListObject = Metadata.getInstance<Select>('checkListLocalAt');
    const valid = component.datasource.data.filter((infos) => (component.value === infos.value));
    const localParentAtObject = Metadata.getInstance<Select>('localParentLocalAt');

    if (valid[0].addInfo === '3') {
      checkListObject.disabled = false;
    } else {
      checkListObject.disabled = true;
    }
    if (valid[0].addInfo === '1') {
      localParentAtObject.disabled = true;
    } else {
      localParentAtObject.disabled = false;
    }
  }

  // Torna visível o field group Tarefa Programada (Não está sendo usado)
  public isVisibleContainer() {
    const containerObject = Metadata.getInstance<Container>('task_container');
    const checkListObject = Metadata.getInstance<Select>('checkListLocalAt');
    const switchObject = Metadata.getInstance<Switch>('switch');

    if (checkListObject.value && switchObject.value) {
      containerObject.isVisible = true;
    } else {
      containerObject.isVisible = false;
    }
  }

  // pega info de uma checkbox e coloca em um ZdText (Não está sendo usado)
  public teste({ component }: any) {
    const itemsText = component.value;
    component.parent.getChildInstance('selected-value').text = `Selected items: ${JSON.stringify(itemsText)}`;
  }

  public consoleLog(variavel: string) {
    const titulo = Metadata.getInstance<Text>('titulo');
    titulo.text = variavel;
  }

  // Form validation
  public createClick() {
    const formObject = Metadata.getInstance<Form>('inputFormLocalAt');
    const formValidate1 = formObject.validate();
    const checkListFormObject = Metadata.getInstance<Form>('checkListForm');
    const formValidate2 = checkListFormObject.validate();
    const localParentAtFormObject = Metadata.getInstance<Form>('localParenteAtForm');
    const formValidate3 = localParentAtFormObject.validate();

    const checkListObject = Metadata.getInstance<Select>('checkListLocalAt');
    const localParentAtObject = Metadata.getInstance<Select>('localParentLocalAt');

    // const taskFormFormObject = Metadata.getInstance<CheckboxMultiple>('week_days_checkBox');
    // const validation = taskFormFormObject.value;
    // const timeObject = Metadata.getInstance<Form>('time_form');
    // const containerObject = Metadata.getInstance<Container>('task_container');
    // const inputTimeObject = Metadata.getInstance<Select>('time');
    // const averageTimeObject = Metadata.getInstance<Select>('averageTime');
    // const employeeObject = Metadata.getInstance<Select>('employee');

    if (checkListObject.disabled === true) {
      checkListObject.removeValidation('required');
      if (localParentAtObject.disabled === true) {
        localParentAtObject.removeValidation('required');
        if (formValidate1) {
          this.insertLocalInfos();
          Router.replace('/gerenciarLocais');
        }
      } else {
        localParentAtObject.addValidation('required');
        if (formValidate1 && formValidate3) {
          this.insertLocalInfos();
          Router.replace('/gerenciarLocais');
        }
      }
    } else {
      checkListObject.addValidation('required');
      if (localParentAtObject.disabled === true) {
        if (formValidate1 && formValidate2) {
          this.insertLocalInfos();
          Router.replace('/gerenciarLocais');
        }
      } else {
        localParentAtObject.addValidation('required');
        if (formValidate1 && formValidate2 && formValidate3) {
          this.insertLocalInfos();
          Router.replace('/gerenciarLocais');
        }
      }
    }
  }

  // Setando valores do campo "Tipo de Local"
  public setLocal() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getLocalType').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.localTypeLocalAt.push(this.setDadosLocal(dados.NMTIPOLOCALCLI, dados.NRTIPOLOCALCLI, dados.IDTIPOLOCALCLI));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Setando valores do campo [LocalParent]
  public setLocalParent() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getLocalParent').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.localParentLocalAt.push(this.setDados(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Setando valores do campo "Setor"
  public setSector() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getSector').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.sectorLocalAt.push(this.setDados(dados.NMSETOR, dados.NRSETOR));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Setando valores do campo "Grupo"
  // public setGroup() {
  //   Http.get('/getGroup').then((response: IDictionary) => {
  //     response.data.dataset.options.forEach((dados: IDictionary) => {
  //       this.localDataList.group.push(this.setDados(dados.NMGRUPOLOCAL, dados.NRGRUPOLOCAL));
  //     });
  //   });
  // }

  // Setando valores do campo "Funcionário"
  // public setEmployee() {
  //   Http.get('/getEmployee').then((response: IDictionary) => {
  //     response.data.dataset.options.forEach((dados: IDictionary) => {
  //       this.localDataList.employee.push(this.setDados(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
  //     });
  //   });
  // }

  // Setando valores do campo "Check List Padrão
  public setCheckList() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getCheckListLocalAtend').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.checkListLocalAt.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Salvando no data do component para exibir na tela
  public findDados({ component }: IEventParam<Select>) {
    component.datasource.data = this.localDataList[component.name];
  }

  /*  --------- Inserts ----------  */

  public insertLocalInfos() {
    const params = {
      NMLOCALCLIENTE: Metadata.getInstance<TextInput>('nameLocalAt').value,
      CDCLIENTE: 'J26269316000177',
      NRTIPOLOCALCLI: Metadata.getInstance<Select>('localTypeLocalAt').value,
      NRLOCALCLIENTEPAI: Metadata.getInstance<Select>('localParentLocalAt').value,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('checkListLocalAt').value,
      NRSETOR: Metadata.getInstance<Select>('sectorLocalAt').value,
      DSICONELOCAL: null,
      DSQRCODE: null,
      NRORG: 20,
      CDOPERINCLUSAO: '000000004280',
      NRORGINCLUSAO: 20,
      DTULTATU: null,
      CDOPERULTATU: null,
      NRORGULTATU: null,
      NRGRUPOLOCAL: null,
    };
    // const test = {
    //   teste: [],
    // };

    // test.teste.push(params);
    Http.post('/insertLocalAt', { params });
  }

  // Inserir tarefa programada (Não está sendo usado)
  // public insertTaskInfos() {
  //   const params = {
  //     NRCHECKLISTAGEND: Metadata.getInstance<Select>('checkListLocalAt').value,
  //     NRFUNCIONARIO: Metadata.getInstance<Select>('employee').value,
  //     DSOBSTAREFAAGEND: null,
  //     NRORG: 20,
  //     CDOPERINCLUSAO: '000000004280',
  //     NRORGINCLUSAO: 20,
  //     DTULTATU: null,
  //     CDOPERULTATU: null,
  //     NRORGULTATU: null,
  //   };

  //   const test = {
  //     teste: [],
  //   };

  //   test.teste.push(params);
  //   // Http.post('/insertTask', { params })
  // }

  /*  --------- Updates ----------  */

  public getToUpdate() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;

    const params = {
      NRLOCALCLIENTE: nr,
    };

    Http.post('/getLocalAtToUpdate', { params }).then((response) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        Metadata.getInstance<Text>('nrLocalAt').text = dados.NRLOCALCLIENTE;
        Metadata.getInstance<TextInput>('nameLocalAt').value = dados.NMLOCALCLIENTE;
        Metadata.getInstance<Select>('localTypeLocalAt').setValue(Number(dados.NRTIPOLOCALCLI));
        Metadata.getInstance<Select>('localParentLocalAt').setValue(dados.NRLOCALCLIENTEPAI);
        Metadata.getInstance<Select>('sectorLocalAt').setValue(dados.NRSETOR);
        Metadata.getInstance<Select>('checkListLocalAt').setValue(dados.NRCHECKLISTPADR);
        this.setUpdate();
      });
    });
  }

  // Muda o label do botao criar e chama atribui o evento de click update para ele
  public setUpdate() {
    const buttonObject = Metadata.getInstance<Button>('saveButtonLocalAt');
    buttonObject.label = 'Salvar';
    buttonObject.events = {
      click: () => this.finalUpdate(),
    };
  }

  // Atualiza os dados do Local de Atendimento
  public finalUpdate() {
    const params = {
      NRLOCALCLIENTE: Metadata.getInstance<Text>('nrLocalAt').text,
      NMLOCALCLIENTE: Metadata.getInstance<TextInput>('nameLocalAt').value,
      NRTIPOLOCALCLI: Metadata.getInstance<Select>('localTypeLocalAt').value,
      NRLOCALCLIENTEPAI: Metadata.getInstance<Select>('localParentLocalAt').value,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('checkListLocalAt').value,
      NRSETOR: Metadata.getInstance<Select>('sectorLocalAt').value,
    };
    Router.replace('/gerenciarLocais');
    Http.post('/updateLocalAt', { params });
  }
}
