import {
  Metadata, IDictionary,
  IEventParam, Router,
  Http, Loader,
} from '@zeedhi/core';
import {
  Select, Form, Text,
  TextInput, Button,
} from '@zeedhi/common';

export class LocalController {
  public dataValue: string = '';

  public editable = false;

  public test = Loader.getInstance('GerenciarLocaisController');

  // Lista para o campo de Check List Padrão
  public checkList: IDictionary = {
    standardCheckList: [],
  };

  // Inicializa os dados buscando dados do back
  public inicializarDados() {
    this.setCheckList();
  }

  // Setando os dados do campo vindos do back
  public setDados(nome: string, code: number) {
    return {
      name: nome,
      value: code,
    };
  }

  // public setCheckList() {
  //   this.checkList.standardCheckList.push(this.setDados('exemplo', 1));
  // }

  // protótipo de card
  get dadosTarefa() {
    return {
      name: 'ColunaCard',
      component: 'ZdCol',
      children: [
        {
          name: 'Cards',
          component: 'ZdCard',
          children: [
            {
              name: 'linhaInterna',
              component: 'ZdRow',
              children: [
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: '<v-icon>mdi-toilet</v-icon> Limpeza de Banheiro',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: 'Banheiro #120 | 3 Andar',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: '26/10/2020 - 06 às 07:00',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: 'Maria Izadora Silva',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '4',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      cssClass: 'zd-justify-end',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: 'Finalizado',
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  // Pegando info de um select
  get selectedItemObject() {
    const selectObject = Metadata.getInstance<Select>('localType');
    const itemsText = (selectObject.value);
    return `${JSON.stringify(itemsText)}`;
  }

  // Habilita o campo checklist
  public validaCheckListPadrao() {
    const localTypeObject = Metadata.getInstance<Select>('localType');
    const checkListObject = Metadata.getInstance<Select>('standardCheckList');
    if (localTypeObject.value != null && localTypeObject.value === 3) {
      checkListObject.disabled = false;
    } else {
      checkListObject.disabled = true;
    }
  }

  // Procurar o pai correspondente
  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  // Forms Validation para botão criar
  public createClick() {
    const formObject = Metadata.getInstance<Form>('input_form');
    const checkListFormObject = Metadata.getInstance<Form>('checkList_form');
    const checkListObject = Metadata.getInstance<Select>('standardCheckList');
    if (formObject.validate()) {
      if (checkListObject.disabled === false) {
        if (checkListFormObject.validate()) {
          this.insertLocalInfos();
          Router.replace('/gerenciarLocais');
        }
      } else {
        this.insertLocalInfos();
        Router.replace('/gerenciarLocais');
      }
    }
  }

  // Salvando no data do component para exibir na tela
  public findDados({ component }: IEventParam<Select>) {
    component.datasource.data = this.checkList[component.name];
  }

  // BackEnd
  public setCheckList() {
    Http.get('/getCheckList').then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        this.checkList.standardCheckList.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
      });
    });
  }

  /*  --------- Insert ----------  */

  public insertLocalInfos() {
    const params = {
      NMTIPOLOCALCLI: Metadata.getInstance<TextInput>('nameInput').value,
      IDTIPOLOCALCLI: Metadata.getInstance<Select>('localType').value,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('standardCheckList').value,
      DSICONETIPOLOCAL: null,
      IDATIVO: 'S',
      NRORG: 20,
      CDOPERINCLUSAO: '000000004280',
      NRORGINCLUSAO: 20,
      DTULTATU: null,
      CDOPERULTATU: null,
      NRORGULTATU: null,
      DSTIPOLOCALCLI: null,
    };

    Http.post('/insertLocal', { params });
  }

  /*  --------- Update ----------  */

  // Pega os dados do tipo de local para update
  public updateLocalInfos({ component }: IEventParam<Button>) {
    // Chamar a tela, mudar de rota para  "/tipolocal"
    const params = {
      NRTIPOLOCALCLI: component.name.substr(3),
    };
    Router.replace('/tipoLocal');
    Http.post('/getLocalToUpdate', { params }).then((response) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        Metadata.getInstance<Text>('nr').text = dados.NRTIPOLOCALCLI;
        Metadata.getInstance<TextInput>('nameInput').value = dados.NMTIPOLOCALCLI;
        const parameter = Metadata.getInstance<Select>('standardCheckList');
        Metadata.getInstance<Select>('localType').setValue(Number(dados.IDTIPOLOCALCLI));
        parameter.setValue(Number(dados.NRCHECKLISTPADR));
        this.setUpdate();
      });
    });
  }

  // Muda o label do botao criar e chama atribui o evento de click update para ele
  public setUpdate() {
    const buttonObject = Metadata.getInstance<Button>('saveButtonLocal');
    buttonObject.label = 'Salvar';
    buttonObject.events = {
      click: () => this.finalUpdate(),
    };
  }

  // Atualiza os dados do tipo de local
  public finalUpdate() {
    const params = {
      NRTIPOLOCALCLI: Metadata.getInstance<Text>('nr').text,
      NMTIPOLOCALCLI: Metadata.getInstance<TextInput>('nameInput').value,
      IDTIPOLOCALCLI: Metadata.getInstance<Select>('localType').value,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('standardCheckList').value,
    };
    Router.replace('/gerenciarLocais');
    Http.post('/updateLocal', { params });
  }
}
