import { Metadata, Http, IDictionary } from '@zeedhi/core';
import { Text, Grid, Progress } from '@zeedhi/common';

export class dashboardController {
  // responsavel por buscar e popular os dados do Grid de limpeza Finalizada
  public selectDadosLimpezaFinalizada() {
    const selectGrigLimpezaFinalizada = Metadata.getInstance<Grid>('grid');
    selectGrigLimpezaFinalizada.datasource.data = [];
    Http.get('/getDadosLimpezaFinalizada').then((response) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        selectGrigLimpezaFinalizada.datasource.data.push({
          Local: dados.NMLOCALCLIENTE,
          TipoLocal: dados.NMLOCALPAI,
          Finalizacao: `Finalizado as ${dados.DTULTATU}`,
        });
      });
    });
  }

  // responsavel por buscar e popular os dados do Grid de Tarefa Atrasada
  public selectDadosTarefaAtrasada() {
    const selectGrigTarefaAtrasada = Metadata.getInstance<Grid>('gridTarefaAtrasada');
    selectGrigTarefaAtrasada.datasource.data = [];
    Http.get('/getDadosTarefaAtrasada').then((response) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        selectGrigTarefaAtrasada.datasource.data.push({
          Local: dados.NMLOCALCLIENTE,
          TipoLocal: dados.NMLOCALPAI,
          Atraso: `Atrasado a ${dados.DTHORAATRASO} horas`,
        });
      });
    });
  }

  // responsavel por buscar e popular os dados do Grid de Tarefa Concluida
  public selectDadosTarefaConcluida() {
    const selectGrigTarefaConcluida = Metadata.getInstance<Grid>('gridTarefaConcluidas');
    selectGrigTarefaConcluida.datasource.data = [];
    Http.get('/getDadosTarefaConcluida').then((response) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        selectGrigTarefaConcluida.datasource.data.push({
          Local: dados.NMLOCALCLIENTE,
          TipoLocal: dados.NMLOCALPAI,
          Concluido: `Concluído as ${dados.DTULTATU} horas`,
        });
      });
    });
  }

  // função para pegar data/hora atual para calculos de atrasado/finalizado e concluido
  public getDataHoraAtual() {
    const horaAtual = new Date();
    const ano = horaAtual.getFullYear();
    const mes = horaAtual.getMonth();
    const dia = horaAtual.getDate();
    const hora = horaAtual.getHours();
    const minutos = horaAtual.getMinutes();
    const segundos = horaAtual.getSeconds();
    const dataHoraAtual = `${dia}-${mes + 1}-${ano} ${hora}:${minutos}:${segundos}`;
    return dataHoraAtual;
  }

  public preencheProgressoDiario() {
    Http.get('/getDadosProgress').then((response) => {
      const data = response.data.dataset.dados.flat(3);
      Metadata.getInstance<Text>('tituloProgress').text = `Tarefas Concluidas: ${data[0].QTDTAREFASCONCLUIDAS}/${data[1].QTDTAREFAS}`;
      const percentual = Math.floor((data[0].QTDTAREFASCONCLUIDAS * 100) / data[1].QTDTAREFAS);
      Metadata.getInstance<Progress>('slotProgress').value = `${percentual}%`;
      Metadata.getInstance<Text>('percentageText').text = `${percentual}%`;
    });
  }

  public atualizaDadosDashboard() {
    setInterval(() => this.selectDadosLimpezaFinalizada(), 120000);
    setInterval(() => this.selectDadosTarefaAtrasada(), 120000);
    setInterval(() => this.selectDadosTarefaConcluida(), 120000);
  }
}
